@@ -22,6 +22,9 @@
 #include "gxdevice.h"
 #include "gxdevsop.h"
 
+/* defined in gsdpram.c */
+int gx_default_get_param(gx_device *dev, char *Param, void *list);
+
 /* ---------------- Default device procedures ---------------- */
 
 /*
@@ -676,6 +679,7 @@ gx_device_fill_in_procs(register gx_device * dev)
     /* NOT put_image */
     fill_dev_proc(dev, dev_spec_op, gx_default_dev_spec_op);
     /* NOT copy_planes */
+    fill_dev_proc(dev, process_page, gx_default_process_page);
 }
 
 int
@@ -693,8 +697,8 @@ void
 gx_default_get_initial_matrix(gx_device * dev, register gs_matrix * pmat)
 {
     /* NB this device has no paper margins */
-    floatp fs_res = dev->HWResolution[0] / 72.0;
-    floatp ss_res = dev->HWResolution[1] / 72.0;
+    double fs_res = dev->HWResolution[0] / 72.0;
+    double ss_res = dev->HWResolution[1] / 72.0;
 
     switch(dev->LeadingEdge & LEADINGEDGE_MASK) {
     case 1: /* 90 degrees */
@@ -918,6 +922,8 @@ int
 gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
 {
     switch(dev_spec_op) {
+        case gxdso_form_begin:
+        case gxdso_form_end:
         case gxdso_pattern_can_accum:
         case gxdso_pattern_start_accum:
         case gxdso_pattern_finish_accum:
@@ -926,9 +932,9 @@ gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
         case gxdso_pattern_is_cpath_accum:
         case gxdso_pattern_handles_clip_path:
         case gxdso_is_pdf14_device:
-        case gxdso_is_native_planar:
         case gxdso_supports_devn:
         case gxdso_supports_hlcolor:
+        case gxdso_supports_saved_pages:
         case gxdso_needs_invariant_palette:
             return 0;
         case gxdso_pattern_shfill_doesnt_need_path:
@@ -955,6 +961,11 @@ gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
                 return 4;
             }
             return 0; /* Otherwise no change */
+        case gxdso_get_dev_param:
+            {
+                dev_param_req_t *request = (dev_param_req_t *)data;
+                return gx_default_get_param(pdev, request->Param, request->list);
+            }
     }
     return gs_error_undefined;
 }
@@ -996,6 +1007,38 @@ gx_default_ret_devn_params(gx_device *pdev)
     return NULL;
 }
 
+int
+gx_default_process_page(gx_device *dev, gx_process_page_options_t *options)
+{
+    gs_int_rect rect;
+    int code = 0;
+    void *buffer = NULL;
+
+    /* Possible future improvements in here could be given by us dividing the
+     * page up into n chunks, and spawning a thread per chunk to do the
+     * process_fn call on. n could be given by NumRenderingThreads. This
+     * would give us multi-core advantages even without clist. */
+    if (options->init_buffer_fn) {
+        code = options->init_buffer_fn(options->arg, dev, dev->memory, dev->width, dev->height, &buffer);
+        if (code < 0)
+            return code;
+    }
+
+    rect.p.x = 0;
+    rect.p.y = 0;
+    rect.q.x = dev->width;
+    rect.q.y = dev->height;
+    if (options->process_fn)
+        code = options->process_fn(options->arg, dev, dev, &rect, buffer);
+    if (code >= 0 && options->output_fn)
+        code = options->output_fn(options->arg, dev, buffer);
+
+    if (options->free_buffer_fn)
+        options->free_buffer_fn(options->arg, dev, dev->memory, buffer);
+
+    return code;
+}
+
 /* ---------------- Default per-instance procedures ---------------- */
 
 int
