@@ -20,6 +20,7 @@
 #include "gp.h"
 #include "gdevdevn.h"           /* for gs_devn_params_s */
 #include "gsdevice.h"		/* for gs_deviceinitialmatrix */
+#include "gxdevsop.h"		/* for gxdso_* */
 #include "gsfname.h"
 #include "gsparam.h"
 #include "gxclio.h"
@@ -130,9 +131,41 @@ gdev_prn_close(gx_device * pdev)
         code = gx_device_close_output_file(pdev, ppdev->fname, ppdev->file);
         ppdev->file = NULL;
     }
+    if (ppdev->saved_pages_list != NULL) {
+        gx_saved_pages_list_free(ppdev->saved_pages_list);
+        ppdev->saved_pages_list = NULL;
+    }
     return code;
 }
 
+int
+gdev_prn_forwarding_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
+{
+    gx_device_printer *ppdev = (gx_device_printer *)pdev;
+
+    if (ppdev->orig_procs.dev_spec_op)
+        return ppdev->orig_procs.dev_spec_op(pdev, dev_spec_op, data, size);
+    return gdev_prn_dev_spec_op(pdev, dev_spec_op, data, size);
+}
+
+int
+gdev_prn_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
+{
+    if (dev_spec_op == gxdso_supports_saved_pages)
+        return 1;
+
+    if (dev_spec_op == gxdso_get_dev_param) {
+        int code;
+        dev_param_req_t *request = (dev_param_req_t *)data;
+
+        code = gdev_prn_get_param(pdev, request->Param, request->list);
+        if (code != gs_error_undefined)
+            return code;
+    }
+
+    return gx_default_dev_spec_op(pdev, dev_spec_op, data, size);
+}
+
 static int		/* returns 0 ok, else -ve error cde */
 gdev_prn_setup_as_command_list(gx_device *pdev, gs_memory_t *buffer_memory,
                                byte **the_memory,
@@ -178,10 +211,12 @@ BACKTRACE(pdev);
 open_c:
     ppdev->buf = base;
     ppdev->buffer_space = space;
+    pclist_dev->common.is_printer = 1;
     clist_init_io_procs(pclist_dev, ppdev->BLS_force_memory);
     clist_init_params(pclist_dev, base, space, pdev,
                       ppdev->printer_procs.buf_procs,
-                      space_params->band, ppdev->is_async_renderer,
+                      space_params->band,
+                      false, /* do_not_open_or_close_bandfiles */
                       (ppdev->bandlist_memory == 0 ? pdev->memory->non_gc_memory:
                        ppdev->bandlist_memory),
                       ppdev->free_up_bandlist_memory,
@@ -350,6 +385,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
             is_command_list = save_is_command_list;
         else {
             is_command_list = space_params.banding_type == BandingAlways ||
+                ppdev->saved_pages_list != NULL ||
                 mem_space >= space_params.MaxBitmap ||
                 !size_ok;	    /* too big to allocate */
         }
@@ -423,7 +459,6 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
                 ppdev->orig_procs.open_device = 0;	/* prevent uninit'd restore of procs */
                 return_error(code);
             }
-            pmemdev->base = base;
         }
         if (ecode == 0)
             break;
@@ -529,6 +564,73 @@ gx_default_close_render_device(gx_device_printer *pdev)
 
 /* ------ Get/put parameters ------ */
 
+int
+gdev_prn_get_param(gx_device *dev, char *Param, void *list)
+{
+    gx_device_printer * const ppdev = (gx_device_printer *)dev;
+    gs_param_list * plist = (gs_param_list *)list;
+    bool pageneutralcolor = false;
+
+    if (strcmp(Param, "Duplex") == 0) {
+        if (ppdev->Duplex_set >= 0) {
+            if (ppdev->Duplex_set)
+                return param_write_bool(plist, "Duplex", &ppdev->Duplex);
+            else
+                return param_write_null(plist, "Duplex");
+        }
+    }
+    if (strcmp(Param, "NumRenderingThreads") == 0) {
+        return param_write_int(plist, "NumRenderingThreads", &ppdev->num_render_threads_requested);
+    }
+    if (strcmp(Param, "OpenOutputFile") == 0) {
+        return param_write_bool(plist, "OpenOutputFile", &ppdev->OpenOutputFile);
+    }
+    if (strcmp(Param, "BGPrint") == 0) {
+        return param_write_bool(plist, "BGPrint", &ppdev->bg_print_requested);
+    }
+    if (strcmp(Param, "ReopenPerPage") == 0) {
+        return param_write_bool(plist, "ReopenPerPage", &ppdev->ReopenPerPage);
+    }
+    if (strcmp(Param, "BandListStorage") == 0) {
+        gs_param_string bls;
+        /* Force the default to 'memory' if clist file I/O is not included in this build */
+        if (clist_io_procs_file_global == NULL)
+            ppdev->BLS_force_memory = true;
+        if (ppdev->BLS_force_memory) {
+            bls.data = (byte *)"memory";
+            bls.size = 6;
+            bls.persistent = false;
+        } else {
+            bls.data = (byte *)"file";
+            bls.size = 4;
+            bls.persistent = false;
+        }
+        return param_write_string(plist, "BandListStorage", &bls);
+    }
+    if (strcmp(Param, "OutputFile") == 0) {
+        gs_param_string ofns;
+
+        ofns.data = (const byte *)ppdev->fname,
+        ofns.size = strlen(ppdev->fname),
+        ofns.persistent = false;
+        return param_write_string(plist, "OutputFile", &ofns);
+    }
+    if (strcmp(Param, "saved-pages") == 0) {
+        gs_param_string saved_pages;
+        /* Always return an empty string for saved-pages */
+        saved_pages.data = (const byte *)"";
+        saved_pages.size = 0;
+        saved_pages.persistent = false;
+        return param_write_string(plist, "saved-pages", &saved_pages);
+    }
+    if (dev->icc_struct != NULL)
+        pageneutralcolor = dev->icc_struct->pageneutralcolor;
+    if (strcmp(Param, "pageneutralcolor") == 0) {
+        return param_write_bool(plist, "pageneutralcolor", &pageneutralcolor);
+    }
+    return gs_error_undefined;
+}
+
 /* Get parameters.  Printer devices add several more parameters */
 /* to the default set. */
 int
@@ -538,22 +640,21 @@ gdev_prn_get_params(gx_device * pdev, gs_param_list * plist)
     int code = gx_default_get_params(pdev, plist);
     gs_param_string ofns;
     gs_param_string bls;
+    gs_param_string saved_pages;
+    bool pageneutralcolor = false;
 
+    if (pdev->icc_struct != NULL)
+        pageneutralcolor = pdev->icc_struct->pageneutralcolor;
     if (code < 0 ||
-        (code = param_write_long(plist, "BandBufferSpace", &ppdev->space_params.band.BandBufferSpace)) < 0 ||
-        (code = param_write_int(plist, "BandHeight", &ppdev->space_params.band.BandHeight)) < 0 ||
-        (code = param_write_int(plist, "BandWidth", &ppdev->space_params.band.BandWidth)) < 0 ||
-        (code = param_write_long(plist, "BufferSpace", &ppdev->space_params.BufferSpace)) < 0 ||
         (ppdev->Duplex_set >= 0 &&
         (code = (ppdev->Duplex_set ?
                   param_write_bool(plist, "Duplex", &ppdev->Duplex) :
                   param_write_null(plist, "Duplex"))) < 0) ||
-        (code = param_write_long(plist, "MaxBitmap", &ppdev->space_params.MaxBitmap)) < 0 ||
         (code = param_write_int(plist, "NumRenderingThreads", &ppdev->num_render_threads_requested)) < 0 ||
         (code = param_write_bool(plist, "OpenOutputFile", &ppdev->OpenOutputFile)) < 0 ||
-        (code = param_write_bool(plist, "PageUsesTransparency", &ppdev->page_uses_transparency)) < 0 ||
         (code = param_write_bool(plist, "BGPrint", &ppdev->bg_print_requested)) < 0 ||
-        (code = param_write_bool(plist, "ReopenPerPage", &ppdev->ReopenPerPage)) < 0
+        (code = param_write_bool(plist, "ReopenPerPage", &ppdev->ReopenPerPage)) < 0 ||
+        (code = param_write_bool(plist, "pageneutralcolor", &pageneutralcolor)) < 0
         )
         return code;
 
@@ -575,7 +676,15 @@ gdev_prn_get_params(gx_device * pdev, gs_param_list * plist)
     ofns.data = (const byte *)ppdev->fname,
         ofns.size = strlen(ppdev->fname),
         ofns.persistent = false;
-    return param_write_string(plist, "OutputFile", &ofns);
+    if ((code = param_write_string(plist, "OutputFile", &ofns)) < 0)
+        return code;
+
+    /* Always return an empty string for saved-pages so that get_params followed */
+    /* by put_params will have no effect.                                       */
+    saved_pages.data = (const byte *)"";
+    saved_pages.size = 0;
+    saved_pages.persistent = false;
+    return param_write_string(plist, "saved-pages", &saved_pages);
 }
 
 /* Validate an OutputFile name by checking any %-formats. */
@@ -600,7 +709,6 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
     bool is_open = pdev->is_open;
     bool oof = ppdev->OpenOutputFile;
     bool rpp = ppdev->ReopenPerPage;
-    bool page_uses_transparency = ppdev->page_uses_transparency;
     bool old_page_uses_transparency = ppdev->page_uses_transparency;
     bool bg_print_requested = ppdev->bg_print_requested;
     bool duplex;
@@ -608,13 +716,15 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
     int width = pdev->width;
     int height = pdev->height;
     int nthreads = ppdev->num_render_threads_requested;
-    gdev_prn_space_params sp, save_sp;
+    gdev_prn_space_params save_sp;
     gs_param_string ofs;
     gs_param_string bls;
     gs_param_dict mdict;
+    gs_param_string saved_pages;
+    bool pageneutralcolor = false;
 
-    sp = ppdev->space_params;
-    save_sp = sp;
+    memset(&saved_pages, 0, sizeof(gs_param_string));
+    save_sp = ppdev->space_params;
 
     switch (code = param_read_bool(plist, (param_name = "OpenOutputFile"), &oof)) {
         default:
@@ -634,16 +744,6 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
             break;
     }
 
-    switch (code = param_read_bool(plist, (param_name = "PageUsesTransparency"),
-                                                        &page_uses_transparency)) {
-        default:
-            ecode = code;
-            param_signal_error(plist, param_name, ecode);
-        case 0:
-        case 1:
-            break;
-    }
-
     if (ppdev->Duplex_set >= 0)	/* i.e., Duplex is supported */
         switch (code = param_read_bool(plist, (param_name = "Duplex"),
                                        &duplex)) {
@@ -660,40 +760,6 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
             case 1:
                 ;
         }
-#define CHECK_PARAM_CASES(member, bad, label)\
-    case 0:\
-        if ((sp.params_are_read_only ? sp.member != save_sp.member : bad))\
-            ecode = gs_error_rangecheck;\
-        else\
-            break;\
-        goto label;\
-    default:\
-        ecode = code;\
-label:\
-        param_signal_error(plist, param_name, ecode);\
-    case 1:\
-        break
-
-    switch (code = param_read_long(plist, (param_name = "MaxBitmap"), &sp.MaxBitmap)) {
-        CHECK_PARAM_CASES(MaxBitmap, sp.MaxBitmap < 0, mbe);
-    }
-
-    switch (code = param_read_long(plist, (param_name = "BufferSpace"), &sp.BufferSpace)) {
-        CHECK_PARAM_CASES(BufferSpace, sp.BufferSpace < 10000, bse);
-    }
-
-    switch (code = param_read_int(plist, (param_name = "BandWidth"), &sp.band.BandWidth)) {
-        CHECK_PARAM_CASES(band.BandWidth, sp.band.BandWidth < 0, bwe);
-    }
-
-    switch (code = param_read_int(plist, (param_name = "BandHeight"), &sp.band.BandHeight)) {
-        CHECK_PARAM_CASES(band.BandHeight, sp.band.BandHeight < 0, bhe);
-    }
-
-    switch (code = param_read_long(plist, (param_name = "BandBufferSpace"), &sp.band.BandBufferSpace)) {
-        CHECK_PARAM_CASES(band.BandBufferSpace, sp.band.BandBufferSpace < 0, bbse);
-    }
-
     switch (code = param_read_string(plist, (param_name = "BandListStorage"), &bls)) {
         case 0:
             /* Only accept 'file' if the file procs are include in the build */
@@ -766,6 +832,25 @@ label:\
             break;
     }
 
+    switch (code = param_read_string(plist, (param_name = "saved-pages"),
+                                                        &saved_pages)) {
+        default:
+            ecode = code;
+            param_signal_error(plist, param_name, ecode);
+        case 0:
+        case 1:
+            break;
+    }
+
+    /* NB: put_params to change pageneutralcolor is allowed, but has no effect */
+    /*     It will be set according the GrayDetection. Allowing it prevents errors */
+    if (pdev->icc_struct != NULL)
+        pageneutralcolor = pdev->icc_struct->pageneutralcolor;
+    if ((code = param_read_bool(plist, (param_name = "pageneutralcolor"),
+                                                        &pageneutralcolor)) < 0) {
+        ecode = code;
+        param_signal_error(plist, param_name, ecode);
+    }
 
     if (ecode < 0)
         return ecode;
@@ -778,13 +863,16 @@ label:\
 
     ppdev->OpenOutputFile = oof;
     ppdev->ReopenPerPage = rpp;
-    ppdev->page_uses_transparency = page_uses_transparency;
+
+    if (ppdev->bg_print_requested && !bg_print_requested) {
+        prn_finish_bg_print(ppdev);
+    }
+    
     ppdev->bg_print_requested = bg_print_requested;
     if (duplex_set >= 0) {
         ppdev->Duplex = duplex;
         ppdev->Duplex_set = duplex_set;
     }
-    ppdev->space_params = sp;
     ppdev->num_render_threads_requested = nthreads;
     if (bls.data != 0) {
         ppdev->BLS_force_memory = (bls.data[0] == 'm');
@@ -794,7 +882,7 @@ label:\
     /* Formerly, would not reallocate if device is not open: */
     /* we had to patch this out (see News for 5.50). */
     code = gdev_prn_maybe_realloc_memory(ppdev, &save_sp, width, height,
-                                                old_page_uses_transparency);
+                                         old_page_uses_transparency);
     if (code < 0)
         return code;
 
@@ -821,6 +909,13 @@ label:\
         if (code < 0)
             return code;
     }
+
+    /* Processing the saved_pages string MAY have side effects, such as printing, */
+    /* allocating or freeing a list. This is (sort of) a write-only parameter, so */
+    /* the get_params will always return an empty string (a no-op action).        */
+    if (saved_pages.data != 0 && saved_pages.size != 0) {
+        return gx_saved_pages_param_process(ppdev, (byte *)saved_pages.data, saved_pages.size);
+    }
     return 0;
 }
 
@@ -838,6 +933,8 @@ gx_default_get_space_params(const gx_device_printer *printer_dev,
 /* If seekable is true, then the printer outputfile must be seekable.            */
 /* If bg_print_ok is true, the device print_page_copies is compatible with the   */
 /* background printing, i.e., thread safe and does not change the device.        */
+/* If the printer device is in 'saved_pages' mode, then background printing is   */
+/* irrelevant and is ignored. In this case, pages are saved to the list.         */
 static int	/* 0 ok, -ve error, or 1 if successfully upgraded to buffer_page */
 gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seekable, bool bg_print_ok)
 {
@@ -845,22 +942,25 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
     gs_devn_params *pdevn_params;
     int outcode = 0, closecode = 0, errcode = 0, endcode;
     bool upgraded_copypage = false;
+    int code;
 
     prn_finish_bg_print(ppdev);		/* finish any previous background printing */
 
-    if (num_copies > 0 || !flush) {
-        int code = gdev_prn_open_printer_seekable(pdev, 1, seekable);
+    if (num_copies > 0 && ppdev->saved_pages_list != NULL) {
+        /* We are putting pages on a list */
+        if ((code = gx_saved_pages_list_add(ppdev)) < 0)
+            return code;
 
-        if (code < 0)
+    } else if (num_copies > 0 || !flush) {
+        if ((code = gdev_prn_open_printer_seekable(pdev, 1, seekable)) < 0)
             return code;
 
         /* If copypage request, try to do it using buffer_page */
         if ( !flush &&
-             (*ppdev->printer_procs.buffer_page)
-             (ppdev, ppdev->file, num_copies) >= 0
-             ) {
+             (*ppdev->printer_procs.buffer_page)(ppdev, ppdev->file, num_copies) >= 0) {
             upgraded_copypage = true;
             flush = true;
+
         } else if (num_copies > 0) {
             int threads_enabled = 0;
             int print_foreground = 1;		/* default to foreground printing */
@@ -881,9 +981,7 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
             while (ppdev->bg_print_requested && threads_enabled) {
                 gx_device *ndev;
                 gx_device_printer *npdev;
-                gx_device_clist_reader *ncrdev;
                 gx_device_clist_reader *crdev = (gx_device_clist_reader *)ppdev;
-                gs_devn_params *pdevn_params;
 
                 if ((code = clist_close_writer_and_init_reader((gx_device_clist *)ppdev)) < 0)
                     /* should not happen -- do foreground print */
@@ -893,14 +991,13 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                     if (((ppdev->bg_print.sema = gx_semaphore_alloc(ppdev->memory->non_gc_memory)) == NULL))
                         break;			/* couldn't create the semaphore */
 
-                ndev = setup_device_and_mem_for_thread(pdev->memory->thread_safe_memory, pdev, true);
+                ndev = setup_device_and_mem_for_thread(pdev->memory->thread_safe_memory, pdev, true, NULL);
                 if (ndev == NULL) {
                     break;
                 }
                 ppdev->bg_print.device = ndev;
                 ppdev->bg_print.num_copies = num_copies;
                 npdev = (gx_device_printer *)ndev;
-                ncrdev = (gx_device_clist_reader *)ndev;
                 npdev->bg_print_requested = 0;
                 npdev->num_render_threads_requested = ppdev->num_render_threads_requested;
 
@@ -954,8 +1051,9 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
         free_separation_names(pdev->memory, &(pdevn_params->separations));
         pdevn_params->num_separation_order_names = 0;
     }
-    endcode = (PRINTER_IS_CLIST(ppdev) && !ppdev->is_async_renderer ?
-               clist_finish_page(pdev, flush) : 0);
+    endcode = (PRINTER_IS_CLIST(ppdev) &&
+              !((gx_device_clist_common *)ppdev)->do_not_open_or_close_bandfiles ?
+              clist_finish_page(pdev, flush) : 0);
 
     if (outcode < 0)
         return outcode;
@@ -1149,8 +1247,8 @@ gdev_prn_open_printer_seekable(gx_device *pdev, bool binary_mode,
 
         if (seekable && !gp_fseekable(ppdev->file)) {
             errprintf(pdev->memory, "I/O Error: Output File \"%s\" must be seekable\n", ppdev->fname);
-            if (ppdev->file != pdev->memory->gs_lib_ctx->fstdout
-              && ppdev->file != pdev->memory->gs_lib_ctx->fstderr) {
+            if (!IS_LIBCTX_STDOUT(pdev->memory, ppdev->file)
+              && !IS_LIBCTX_STDERR(pdev->memory ,ppdev->file)) {
 
                 code = gx_device_close_output_file(pdev, ppdev->fname, ppdev->file);
                 if (code < 0)
@@ -1274,8 +1372,12 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target, int y,
 
     if (plane_index >= 0)
         depth = render_plane->depth;
-    else
+    else {
         depth = target->color_info.depth;
+        if (target->is_planar)
+            depth /= target->color_info.num_components;
+    }
+
     mdproto = gdev_mem_device_for_bits(depth);
     if (mdproto == 0)
         return_error(gs_error_rangecheck);
@@ -1291,13 +1393,25 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target, int y,
         /* The following is a special hack for setting up printer devices. */
         assign_dev_procs(mdev, mdproto);
         check_device_separable((gx_device *)mdev);
+        /* In order for saved-pages to work, we need to hook the dev_spec_op */
+        if (mdev->procs.dev_spec_op == NULL)
+            set_dev_proc(mdev, dev_spec_op, gdev_prn_dev_spec_op);
+#ifdef DEBUG
+        /* scanning sources didn't show anything, but if a device gets changed or added */
+        /* that has its own dev_spec_op, it should call the gdev_prn_spec_op as well    */
+        else
+            errprintf(mdev->memory, "Warning: printer device has private dev_spec_op\n");
+#endif
         gx_device_fill_in_procs((gx_device *)mdev);
     } else {
         gs_make_mem_device(mdev, mdproto, mem, (color_usage == NULL ? 1 : 0),
-                           (target == (gx_device *)mdev ? NULL : target));
+                           target);
     }
     mdev->width = target->width;
     mdev->band_y = y;
+    mdev->log2_align_mod = target->log2_align_mod;
+    mdev->pad = target->pad;
+    mdev->is_planar = target->is_planar;
     /*
      * The matrix in the memory device is irrelevant,
      * because all we do with the device is call the device-level
@@ -1342,7 +1456,9 @@ gx_default_size_buf_device(gx_device_buf_space_t *space, gx_device *target,
         (render_plane && render_plane->index >= 0 ? render_plane->depth :
          target->color_info.depth);
     mdev.width = target->width;
-    mdev.num_planes = 0;
+    mdev.is_planar = target->is_planar;
+    mdev.pad = target->pad;
+    mdev.log2_align_mod = target->log2_align_mod;
     if (gdev_mem_bits_size(&mdev, target->width, height, &(space->bits)) < 0)
         return_error(gs_error_VMerror);
     space->line_ptrs = gdev_mem_line_ptrs_size(&mdev, target->width, height);
@@ -1352,7 +1468,7 @@ gx_default_size_buf_device(gx_device_buf_space_t *space, gx_device *target,
 
 /* Set up the buffer device. */
 int
-gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
+gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int raster,
                             byte **line_ptrs, int y, int setup_height,
                             int full_height)
 {
@@ -1360,12 +1476,8 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
         (gs_device_is_memory(bdev) ? (gx_device_memory *)bdev :
          (gx_device_memory *)(((gx_device_plane_extract *)bdev)->plane_dev));
     byte **ptrs = line_ptrs;
-    int raster = bytes_per_line;
     int code;
 
-    /****** HACK ******/
-    if ((gx_device *)mdev == bdev && mdev->num_planes)
-        raster = bitmap_raster(mdev->planes[0].depth * mdev->width);
     if (ptrs == 0) {
         /*
          * Before allocating a new line pointer array, if there is a previous
@@ -1381,8 +1493,8 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
          */
         ptrs = (byte **)
             gs_alloc_byte_array(mdev->memory,
-                                (mdev->num_planes ?
-                                 full_height * mdev->num_planes :
+                                (mdev->is_planar ?
+                                 full_height * mdev->color_info.num_components :
                                  setup_height),
                                 sizeof(byte *), "setup_buf_device");
         if (ptrs == 0)
@@ -1391,7 +1503,7 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
         mdev->foreign_line_pointers = false;
     }
     mdev->height = full_height;
-    code = gdev_mem_set_line_ptrs(mdev, buffer + raster * y, bytes_per_line,
+    code = gdev_mem_set_line_ptrs(mdev, buffer + raster * y, raster,
                                   ptrs, setup_height);
     mdev->height = setup_height;
     bdev->height = setup_height; /* do here in case mdev == bdev */
@@ -1536,8 +1648,6 @@ compare_gdev_prn_space_params(const gdev_prn_space_params sp1,
     return(1);
   if (sp1.BufferSpace != sp2.BufferSpace)
     return(1);
-  if (sp1.band.page_uses_transparency != sp2.band.page_uses_transparency)
-    return(1);
   if (sp1.band.BandWidth != sp2.band.BandWidth)
     return(1);
   if (sp1.band.BandHeight != sp2.band.BandHeight)
