@@ -84,10 +84,22 @@ gs_setcolor(gs_state * pgs, const gs_client_color * pcc)
 {
     gs_color_space *    pcs = pgs->color[0].color_space;
     gs_client_color     cc_old = *pgs->color[0].ccolor;
+    gx_device_color *dev_color = pgs->color[0].dev_color;
+    bool do_unset  = true;
 
-   if (pgs->in_cachedevice)
+    if (pgs->in_cachedevice)
         return_error(gs_error_undefined); /* PLRM3 page 215. */
-    gx_unset_dev_color(pgs);
+    if (dev_color->ccolor_valid && gx_dc_is_pure(dev_color)) {      /* change of colorspace will set type to _none */
+        int i;
+        int ncomps = cs_num_components(pcs);
+
+        for(i=0; i < ncomps; i++)
+            if (dev_color->ccolor.paint.values[i] != pcc->paint.values[i])
+                break;
+        do_unset = i < ncomps;      /* if i == ncomps, color unchanged, optimized */
+    }
+    if (do_unset)
+        gx_unset_dev_color(pgs);
     (*pcs->type->adjust_color_count)(pcc, pcs, 1);
     *pgs->color[0].ccolor = *pcc;
     (*pcs->type->restrict_color)(pgs->color[0].ccolor, pcs);
@@ -177,6 +189,7 @@ static cs_proc_install_cspace(gx_install_Indexed);
 static cs_proc_set_overprint(gx_set_overprint_Indexed);
 static cs_proc_final(gx_final_Indexed);
 static cs_proc_serialize(gx_serialize_Indexed);
+static cs_proc_polarity(gx_polarity_Indexed);
 const gs_color_space_type gs_color_space_type_Indexed = {
     gs_color_space_index_Indexed, false, false,
     &st_color_space_Indexed, gx_num_components_1,
@@ -188,7 +201,7 @@ const gs_color_space_type gs_color_space_type_Indexed = {
     gx_set_overprint_Indexed,
     gx_final_Indexed, gx_no_adjust_color_count,
     gx_serialize_Indexed,
-    gx_cspace_is_linear_default
+    gx_cspace_is_linear_default, gx_polarity_Indexed
 };
 
 /* GC procedures. */
@@ -224,6 +237,14 @@ RELOC_PTRS_END
 
 /* Color space installation for an Indexed color space. */
 
+/* Return polarity of base space */
+static gx_color_polarity_t
+gx_polarity_Indexed(const gs_color_space * pcs)
+{
+    return (*pcs->base_space->type->polarity)
+        ((const gs_color_space *)pcs->base_space);
+}
+
 static int
 gx_install_Indexed(gs_color_space * pcs, gs_state * pgs)
 {
