@@ -102,7 +102,7 @@ tile_fill_init(tile_fill_state_t * ptfs, const gx_device_color * pdevc,
     bool is_planar;
 
     ptfs->pdevc = pdevc;
-    is_planar = dev_proc(dev, dev_spec_op)(dev, gxdso_is_native_planar, NULL, 0) > 0;
+    is_planar = dev->is_planar;
     if (is_planar) {
         ptfs->num_planes = dev->color_info.num_components;
     } else {
@@ -334,7 +334,7 @@ tile_pattern_clist(const tile_fill_state_t * ptfs,
         code = clist_read_icctable(crdev);
     /* Also allocate the icc cache for the clist reader */
     if ( crdev->icc_cache_cl == NULL )
-        crdev->icc_cache_cl = gsicc_cache_new(crdev->memory);
+        crdev->icc_cache_cl = gsicc_cache_new(crdev->memory->thread_safe_memory);
     if_debug0m('L', dev->memory, "Pattern clist playback begin\n");
     code = clist_playback_file_bands(playback_action_render,
                 crdev, &crdev->page_info, dev, 0, 0, ptfs->xoff - x, ptfs->yoff - y);
@@ -946,16 +946,24 @@ gx_dc_pat_trans_fill_rectangle(const gx_device_color * pdevc, int x, int y,
     phase.x = pdevc->phase.x;
     phase.y = pdevc->phase.y;
 
+#if 0
+    if_debug8m('v', ptile->ttrans->mem,
+            "[v]gx_dc_pat_trans_fill_rectangle, Fill: (%d, %d), %d x %d To Buffer: (%d, %d), %d x %d \n",
+            x, y, w, h,  ptile->ttrans->fill_trans_buffer->rect.p.x, 
+            ptile->ttrans->fill_trans_buffer->rect.p.y, 
+            ptile->ttrans->fill_trans_buffer->rect.q.x - 
+            ptile->ttrans->fill_trans_buffer->rect.p.x,
+            ptile->ttrans->fill_trans_buffer->rect.q.y - 
+            ptile->ttrans->fill_trans_buffer->rect.p.y);
+#endif
     code = gx_trans_pattern_fill_rect(x, y, x+w, y+h, ptile,
                                       ptile->ttrans->fill_trans_buffer, phase,
                                       dev, pdevc);
-
     return code;
 }
 
 /* This fills the transparency buffer rectangles with a pattern buffer
    that includes transparency */
-
 int
 gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
                            gx_color_tile *ptile,
@@ -963,21 +971,20 @@ gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
                            gs_int_point phase, gx_device *dev,
                            const gx_device_color * pdevc)
 {
-
     tile_fill_trans_state_t state_trans;
     tile_fill_state_t state_clist_trans;
     int code = 0;
+    int w = xmax - xmin;
+    int h = ymax - ymin;
 
     if (ptile == 0)             /* null pattern */
         return 0;
-
-    /* Fit fill */
-    if ( (xmin | ymin) < 0 ) {
-        if ( xmin < 0 )
-            xmin = 0;
-        if ( ymin < 0 )
-            ymin = 0;
-    }
+   
+    fit_fill_xywh(dev, xmin, ymin, w, h);
+    if (w < 0 || h < 0) 
+        return 0;
+    xmax = w + xmin;
+    ymax = h + ymin;
 
     /* Initialize the fill state */
     state_trans.phase.x = phase.x;
@@ -1029,8 +1036,8 @@ gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
             tbits.size.x = crdev->width;
             tbits.size.y = crdev->height;
             if (code >= 0)
-                code = tile_by_steps(&state_clist_trans, xmin, ymin, xmax,
-                                     ymax, ptile, &tbits, tile_pattern_clist);
+                code = tile_by_steps(&state_clist_trans, xmin, ymin, xmax-xmin,
+                                     ymax-ymin, ptile, &tbits, tile_pattern_clist);
 
             if (code >= 0 && (state_clist_trans.cdev != NULL)) {
                 tile_clip_free((gx_device_tile_clip *)state_clist_trans.cdev);
@@ -1039,5 +1046,5 @@ gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
 
         }
     }
-    return(code);
+    return code;
 }
