@@ -28,6 +28,8 @@
 #include "gdevpdfc.h"
 #include "gdevpdfo.h"
 #include "strimpl.h"
+#include "gsicc_create.h"
+#include "gsicc_manage.h"
 
 /* ------ CIE space synthesis ------ */
 
@@ -41,7 +43,7 @@ pdf_cie_add_ranges(cos_dict_t *pcd, const gs_range *prange, int n, bool clamp)
     if (pca == 0)
         return_error(gs_error_VMerror);
     for (i = 0; i < n; ++i) {
-        floatp rmin = prange[i].rmin, rmax = prange[i].rmax;
+        double rmin = prange[i].rmin, rmax = prange[i].rmax;
 
         if (clamp) {
             if (rmin < 0) rmin = 0;
@@ -73,7 +75,7 @@ cie_to_xyz(const double *in, double out[3], const gs_color_space *pcs,
 
     gs_color_space_index cs_index;
     const gs_vector3 *const pWhitePoint = &pciec->points.WhitePoint;
-    double xyz_float[3];
+    float xyz_float[3];
 
     cs_index = gs_color_space_get_index(pcs);
     /* Need a device profile */
@@ -100,16 +102,16 @@ cie_to_xyz(const double *in, double out[3], const gs_color_space *pcs,
 
     switch (cs_index) {
         case gs_color_space_index_CIEA:
-            gx_psconcretize_CIEA(&cc, pcs, xyz, pis);
+            gx_psconcretize_CIEA(&cc, pcs, xyz, xyz_float, pis);
             break;
         case gs_color_space_index_CIEABC:
-            gx_psconcretize_CIEABC(&cc, pcs, xyz, pis);
+            gx_psconcretize_CIEABC(&cc, pcs, xyz, xyz_float, pis);
             break;
         case gs_color_space_index_CIEDEF:
-            gx_psconcretize_CIEDEF(&cc, pcs, xyz, pis);
+            gx_psconcretize_CIEDEF(&cc, pcs, xyz, xyz_float, pis);
             break;
         case gs_color_space_index_CIEDEFG:
-           gx_psconcretize_CIEDEFG(&cc, pcs, xyz, pis);
+           gx_psconcretize_CIEDEFG(&cc, pcs, xyz, xyz_float, pis);
            break;
         default:
             break;
@@ -121,14 +123,9 @@ cie_to_xyz(const double *in, double out[3], const gs_color_space *pcs,
         /* Use the resulting Y value to scale the wp Illumination.
         note that we scale to the whitepoint here.  Matrix out
         handles mapping to CIE D50.  This forces an achromatic result */
-        xyz_float[1] = frac2float(xyz[1]);
         xyz_float[0] = pWhitePoint->u * xyz_float[1];
         xyz_float[2] = pWhitePoint->w * xyz_float[1];
-    } else {
-        xyz_float[0] = frac2float(xyz[0]);
-        xyz_float[1] = frac2float(xyz[1]);
-        xyz_float[2] = frac2float(xyz[2]);
-    }
+    } 
 
     /* Do wp mapping to D50 in XYZ for now.  We should do bradford correction.
        Will add that in next release */
@@ -256,7 +253,8 @@ pdf_convert_cie_to_lab(gx_device_pdf *pdev, cos_array_t *pca,
  * the profile data on *ppcstrm.
  */
 static int
-pdf_make_iccbased(gx_device_pdf *pdev, cos_array_t *pca, int ncomps,
+pdf_make_iccbased(gx_device_pdf *pdev, const gs_imager_state * pis,
+                  cos_array_t *pca, int ncomps,
                   const gs_range *prange /*[4]*/,
                   const gs_color_space *pcs_alt,
                   cos_stream_t **ppcstrm,
@@ -333,14 +331,22 @@ pdf_make_iccbased(gx_device_pdf *pdev, cos_array_t *pca, int ncomps,
         case gs_color_space_index_DeviceCMYK:
             break;			/* implicit (default) */
         default:
-            if ((code = pdf_color_space_named(pdev, &v, NULL, pcs_alt,
-                                        &pdf_color_space_names, false, NULL, 0)) < 0 ||
+            if ((code = pdf_color_space_named(pdev, pis, &v, NULL, pcs_alt,
+                                        &pdf_color_space_names, false, NULL, 0, true)) < 0 ||
                 (code = cos_dict_put_c_key(cos_stream_dict(pcstrm), "/Alternate",
                                            &v)) < 0
                 )
                 goto fail;
         }
 
+    } else {
+        if (ncomps != 1 && ncomps != 3 && ncomps != 4) {
+            /* We can only use a default for Gray, RGB or CMYK. For anything else we need
+             * to convert to the base space, we can't legally preserve the ICC profile.
+             */
+            code = gs_error_rangecheck;
+            goto fail;
+        }
     }
 
     /* Wrap up. */
@@ -409,7 +415,7 @@ set_uint32(byte bytes[4], uint value)
     bytes[3] = (byte)value;
 }
 static void
-set_XYZ(byte bytes[4], floatp value)
+set_XYZ(byte bytes[4], double value)
 {
     set_uint32(bytes, (uint)(int)(value * 65536));
 }
@@ -424,7 +430,7 @@ add_table_xyz3(profile_table_t **ppnt, const char *tag, byte bytes[20],
     DISCARD(add_table(ppnt, tag, bytes, 20));
 }
 static void
-set_sample16(byte *p, floatp v)
+set_sample16(byte *p, double v)
 {
     int value = (int)(v * 65535);
 
@@ -708,7 +714,7 @@ pdf_convert_cie_to_iccbased(gx_device_pdf *pdev, cos_array_t *pca,
     white_d50.w = 0.8249f;
 
     pdf_cspace_init_Device(pdev->memory, &alt_space, ncomps);	/* can't fail */
-    code = pdf_make_iccbased(pdev, pca, ncomps, prange, alt_space,
+    code = pdf_make_iccbased(pdev, NULL, pca, ncomps, prange, alt_space,
                              &pcstrm, pprange);
     rc_decrement_cs(alt_space, "pdf_convert_cie_to_iccbased");
     if (code < 0)
@@ -812,7 +818,7 @@ pdf_convert_cie_to_iccbased(gx_device_pdf *pdev, cos_array_t *pca,
  * broken out only for readability.
  */
 int
-pdf_iccbased_color_space(gx_device_pdf *pdev, cos_value_t *pvalue,
+pdf_iccbased_color_space(gx_device_pdf *pdev, const gs_imager_state * pis, cos_value_t *pvalue,
                          const gs_color_space *pcs, cos_array_t *pca)
 {
     /*
@@ -821,7 +827,7 @@ pdf_iccbased_color_space(gx_device_pdf *pdev, cos_value_t *pvalue,
      */
     cos_stream_t * pcstrm;
     int code =
-        pdf_make_iccbased(pdev, pca, pcs->cmm_icc_profile_data->num_comps,
+        pdf_make_iccbased(pdev, pis, pca, pcs->cmm_icc_profile_data->num_comps,
                           pcs->cmm_icc_profile_data->Range.ranges,
                           pcs->base_space,
                           &pcstrm, NULL);
@@ -831,8 +837,23 @@ pdf_iccbased_color_space(gx_device_pdf *pdev, cos_value_t *pvalue,
 
     /* Transfer the buffer data  */
 
-    code = cos_stream_add_bytes(pcstrm, pcs->cmm_icc_profile_data->buffer,
+    /* PDF/A-1 only supports version 2 ICC profiles, and the PDF spec
+     * only supports Version 4 profiles in PDF 1.5 and above
+     */
+    if (pdev->CompatibilityLevel < 1.5 || pdev->PDFA == 1) {
+        byte *v2_buffer;
+        int size;
+
+        if (pis == NULL)
+            return (gs_error_undefined);
+        if (pcs->cmm_icc_profile_data->profile_handle == NULL)
+            gsicc_initialize_default_profile(pcs->cmm_icc_profile_data);
+        v2_buffer = gsicc_create_getv2buffer(pis, pcs->cmm_icc_profile_data, &size);
+        code = cos_stream_add_bytes(pcstrm, v2_buffer, size);
+    }else{
+        code = cos_stream_add_bytes(pcstrm, pcs->cmm_icc_profile_data->buffer,
         pcs->cmm_icc_profile_data->buffer_size);
+    }
 
     if (code >= 0)
         code = pdf_finish_iccbased(pcstrm);
