@@ -15,6 +15,8 @@
 
 
 /* Common support for interpreter front ends */
+
+
 #include "malloc_.h"
 #include "memory_.h"
 #include "string_.h"
@@ -24,7 +26,11 @@
 #include "gslib.h"
 #include "gsmatrix.h"           /* for gxdevice.h */
 #include "gsutil.h"             /* for bytes_compare */
+#include "gspaint.h"		/* for gs_erasepage */
 #include "gxdevice.h"
+#include "gxdevsop.h"		/* for gxdso_* enums */
+#include "gxclpage.h"
+#include "gdevprn.h"
 #include "gxalloc.h"
 #include "gxiodev.h"            /* for iodev struct */
 #include "gzstate.h"
@@ -315,6 +321,7 @@ gs_main_init2(gs_main_instance * minst)
 {
     i_ctx_t *i_ctx_p;
     int code = gs_main_init1(minst);
+    int initial_init_level = minst->init_done;
 
     if (code < 0)
         return code;
@@ -323,6 +330,38 @@ gs_main_init2(gs_main_instance * minst)
     if (code < 0)
        return code;
     i_ctx_p = minst->i_ctx_p; /* display_set_callback or run_string may change it */
+
+    /* Now process the initial saved-pages=... argument, if any as well as saved-pages-test */
+    if (initial_init_level < 2) {
+       gx_device *pdev = gs_currentdevice(minst->i_ctx_p->pgs);	/* get the current device */
+       gx_device_printer *ppdev = (gx_device_printer *)pdev;
+
+        if (minst->saved_pages_test_mode) {
+            if ((dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0)) {
+                /* no warning or error if saved-pages-test mode is used, just disable it */
+                minst->saved_pages_test_mode = false;  /* device doesn't support it */
+            } else {
+                if ((code = gx_saved_pages_param_process(ppdev, (byte *)"begin", 5)) < 0)
+                    return code;
+                if (code > 0)
+                    if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                        return code;
+            }
+        } else if (minst->saved_pages_initial_arg != NULL) {
+            if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0) {
+                outprintf(minst->heap,
+                          "   --saved-pages not supported by the '%s' device.\n",
+                          pdev->dname);
+                return e_Fatal;
+            }
+            code = gx_saved_pages_param_process(ppdev, minst->saved_pages_initial_arg,
+                                                strlen(minst->saved_pages_initial_arg));
+            if (code > 0)
+                if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                    return code;
+        }
+    }
+
     if (gs_debug_c(':'))
         print_resource_usage(minst, &gs_imemory, "Start");
     gp_readline_init(&minst->readline_data, imemory_system);
@@ -657,7 +696,7 @@ gs_push_integer(gs_main_instance * minst, long value)
 }
 
 int
-gs_push_real(gs_main_instance * minst, floatp value)
+gs_push_real(gs_main_instance * minst, double value)
 {
     ref vref;
 
@@ -833,6 +872,25 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      * alloc_restore_all will close dynamically allocated devices.
      */
     tempnames = gs_main_tempnames(minst);
+
+#ifndef PSI_INCLUDED
+    /* We have to disable BGPrint before we call interp_reclaim() to prevent the
+     * parent rendering thread initialising for the next page, whilst we are
+     * removing objects it may want to access - for example, the I/O device table.
+     * We also have to mess with the BeginPage/EndPage procs so that we don't
+     * trigger a spurious extra page to be emitted.
+     */
+    if (minst->init_done >= 1) {
+        gs_main_run_string(minst,
+            "/systemdict .systemexec /begin .systemexec \
+            <</BeginPage {pop} /EndPage {pop pop //false } \
+              /BGPrint false /NumRenderingThreads 0>> setpagedevice \
+              serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse end \
+              .systemvar exec",
+            0 , &exit_code, &error_object);
+    }
+#endif
+
     /*
      * Close the "main" device, because it may need to write out
      * data before destruction. pdfwrite needs so.
@@ -861,8 +919,8 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
             /* deactivate the device just before we close it for the last time */
             gs_main_run_string(minst,
                 /* we need to do the 'quit' so we don't loop for input (double quit) */
-                ".uninstallpagedevice "
-                "serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
+                ".uninstallpagedevice serverdict \
+                /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
                 0 , &exit_code, &error_object);
             code = gs_closedevice(pdev);
             if (code < 0)
@@ -879,8 +937,10 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     /* Flush stdout and stderr */
     if (minst->init_done >= 2)
       gs_main_run_string(minst,
-        "(%stdout) (w) file closefile (%stderr) (w) file closefile "
-        "serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
+        "(%stdout) (w) file closefile (%stderr) (w) file closefile \
+        /systemdict .systemexec /begin .systemexec \
+        serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemexec \
+        end",
         0 , &exit_code, &error_object);
     gp_readline_finit(minst->readline_data);
     i_ctx_p = minst->i_ctx_p;		/* get current interp context */
