@@ -34,6 +34,7 @@
 #include "gxgetbit.h"
 #include "store.h"
 #include "gsicc_manage.h"
+#include "gxdevsop.h"
 
 /* <device> <keep_open> .copydevice2 <newdevice> */
 static int
@@ -217,7 +218,7 @@ zgetdefaultdevice(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     const gx_device *dev;
 
-    dev = gs_getdefaultdevice();
+    dev = gs_getdefaultlibdevice(imemory);
     if (dev == 0) /* couldn't find a default device */
         return_error(e_unknownerror);
     push(1);
@@ -470,6 +471,7 @@ zsetdevice(i_ctx_t *i_ctx_p)
         if(op->value.pdevice != dev) 	  /* don't allow a different device    */
             return_error(e_invalidaccess);
     }
+    dev->ShowpageCount = 0;
 #ifndef PSI_INCLUDED
     /* the language switching build shouldn't install a new device
        here.  The language switching machinery installs a shared
@@ -489,6 +491,129 @@ zsetdevice(i_ctx_t *i_ctx_p)
     return code;
 }
 
+/* Custom PostScript operator '.special_op' is used to implement
+ * 'dev_spec_op' access from PostScript. Initially this is intended
+ * to be used to recover individual device parameters from certain
+ * devices (pdfwrite, ps2write etc). In the future we may choose to
+ * increase the devices which can support this, and make more types
+ * of 'spec_op' available from the PostScript world.
+ */
+
+/* We use this structure in a table below which allows us to add new
+ * 'spec_op's with minimum fuss.
+ */
+typedef struct spec_op_s spec_op_t;
+struct spec_op_s {
+    char *name;					/* C string representing the name of the spec_op */
+    int spec_op;                /* Integer used to switch on the name */
+};
+
+/* To ad    d more spec_ops, put a key name (used to identify the specific
+ * spec_op required) in this table, the integer is just used in the switch
+ * in the main code to execute the required spec_op code.
+ */
+spec_op_t spec_op_defs[] = {
+    {(char *)"GetDeviceParam", 0}
+};
+
+/* <any> <any> .... /spec_op name .special_op <any> <any> .....
+ * The special_op operator takes at a minimum the name of the spec_op to execute
+ * and as many additional parameters as are required for the spec_op. It may
+ * return as many additional parameters as required.
+ */
+int
+zspec_op(i_ctx_t *i_ctx_p)
+{
+    os_ptr  op = osp;
+    gx_device *dev = gs_currentdevice(igs);
+    int i, nprocs = sizeof(spec_op_defs) / sizeof(spec_op_t), code, proc = -1;
+    ref opname, nref, namestr;
+    char *data;
+
+    /* At the very minimum we need a name object telling us which sepc_op to perform */
+    check_op(1);
+    if (!r_has_type(op, t_name))
+        return_error(e_typecheck);
+
+    ref_assign(&opname, op);
+
+    /* Find the relevant spec_op name */
+    for (i=0;i<nprocs;i++) {
+        code = names_ref(imemory->gs_lib_ctx->gs_name_table, (const byte *)spec_op_defs[i].name, strlen(spec_op_defs[i].name), &nref, 0);
+        if (code < 0)
+            return code;
+        if (name_eq(&opname, &nref)) {
+            proc = i;
+            break;
+        }
+    }
+
+    if (proc < 0)
+        return_error(e_undefined);
+
+    pop(1);     /* We don't need the name of the spec_op any more */
+    op = osp;
+
+    switch(proc) {
+        case 0:
+            {
+                stack_param_list list;
+                dev_param_req_t request;
+                ref rkeys;
+                /* Get a single device parameter, we should be supplied with
+                 * the name of the paramter, as a name object.
+                 */
+                check_op(1);
+                if (!r_has_type(op, t_name))
+                    return_error(e_typecheck);
+
+                ref_assign(&opname, op);
+                name_string_ref(imemory, &opname, &namestr);
+
+                data = (char *)gs_alloc_bytes(imemory, r_size(&namestr) + 1, "temporary special_op string");
+                if (data == 0)
+                    return_error(e_VMerror);
+                memset(data, 0x00, r_size(&namestr) + 1);
+                memcpy(data, namestr.value.bytes, r_size(&namestr));
+
+                /* Discard the parameter name now, we're done with it */
+                pop (1);
+                /* Make a null object so that the stack param list won't check for requests */
+                make_null(&rkeys);
+                stack_param_list_write(&list, &o_stack, &rkeys, iimemory);
+                /* Stuff the data into a structure for passing to the spec_op */
+                request.Param = data;
+                request.list = &list;
+
+                code = dev_proc(dev, dev_spec_op)(dev, gxdso_get_dev_param, &request, sizeof(dev_param_req_t));
+
+                gs_free_object(imemory, data, "temporary special_op string");
+
+                if (code < 0) {
+                    if (code == gs_error_undefined) {
+                        op = osp;
+                        push(1);
+                        make_bool(op, 0);
+                    } else
+                        return_error(code);
+                } else {
+                    op = osp;
+                    push(1);
+                    make_bool(op, 1);
+                }
+            }
+            break;
+        default:
+            /* Belt and braces; it shold not be possible to get here, as the table
+             * containing the names should mirror the entries in this switch. If we
+             * found a name there should be a matching case here.
+             */
+            return_error(e_undefined);
+            break;
+    }
+    return 0;
+}
+
 /* ------ Initialization procedure ------ */
 
 const op_def zdevice_op_defs[] =
@@ -510,3 +635,10 @@ const op_def zdevice_op_defs[] =
     {"1.setdevice", zsetdevice},
     op_def_end(0)
 };
+
+/* We need to split the table because of the 16-element limit. */
+const op_def zdevice_ext_op_defs[] =
+{
+    {"0.special_op", zspec_op},
+    op_def_end(0)
+};
